@@ -33,7 +33,7 @@
 #include "nsTextFormatter.h"
 #include "mozilla/Services.h"
 
-static PRLogModuleInfo * gMimeEmitterLogModule = nsnull;
+static PRLogModuleInfo * gMimeEmitterLogModule = nullptr;
 
 #define   MIME_HEADER_URL      "chrome://messenger/locale/mimeheader.properties"
 #define   MIME_URL             "chrome://messenger/locale/mime.properties"
@@ -52,29 +52,29 @@ nsMimeBaseEmitter::nsMimeBaseEmitter()
   // Initialize data output vars...
   mFirstHeaders = true;
 
-  mBufferMgr = nsnull;
+  mBufferMgr = nullptr;
   mTotalWritten = 0;
   mTotalRead = 0;
-  mInputStream = nsnull;
-  mOutStream = nsnull;
-  mOutListener = nsnull;
+  mInputStream = nullptr;
+  mOutStream = nullptr;
+  mOutListener = nullptr;
 
   // Display output control vars...
   mDocHeader = false;
-  m_stringBundle = nsnull;
-  mURL = nsnull;
+  m_stringBundle = nullptr;
+  mURL = nullptr;
   mHeaderDisplayType = nsMimeHeaderDisplayTypes::NormalHeaders;
 
   // Setup array for attachments
   mAttachCount = 0;
   mAttachArray = new nsVoidArray();
-  mCurrentAttachment = nsnull;
+  mCurrentAttachment = nullptr;
 
   // Header cache...
   mHeaderArray = new nsVoidArray();
 
   // Embedded Header Cache...
-  mEmbeddedHeaderArray = nsnull;
+  mEmbeddedHeaderArray = nullptr;
 
   // HTML Header Data...
 //  mHTMLHeaders = "";
@@ -98,13 +98,13 @@ nsMimeBaseEmitter::nsMimeBaseEmitter()
 
 nsMimeBaseEmitter::~nsMimeBaseEmitter(void)
 {
-  PRInt32 i;
+  int32_t i;
 
   // Delete the buffer manager...
   if (mBufferMgr)
   {
     delete mBufferMgr;
-    mBufferMgr = nsnull;
+    mBufferMgr = nullptr;
   }
 
   // Clean up the attachment array structures...
@@ -127,10 +127,10 @@ nsMimeBaseEmitter::~nsMimeBaseEmitter(void)
 
   // Cleanup allocated header arrays...
   CleanupHeaderArray(mHeaderArray);
-  mHeaderArray = nsnull;
+  mHeaderArray = nullptr;
 
   CleanupHeaderArray(mEmbeddedHeaderArray);
-  mEmbeddedHeaderArray = nsnull;
+  mEmbeddedHeaderArray = nullptr;
 }
 
 NS_IMETHODIMP nsMimeBaseEmitter::GetInterface(const nsIID & aIID, void * *aInstancePtr)
@@ -145,7 +145,7 @@ nsMimeBaseEmitter::CleanupHeaderArray(nsVoidArray *aArray)
   if (!aArray)
     return;
 
-  for (PRInt32 i=0; i<aArray->Count(); i++)
+  for (int32_t i=0; i<aArray->Count(); i++)
   {
     headerInfoType *headerInfo = (headerInfoType *)aArray->ElementAt(i);
     if (!headerInfo)
@@ -159,7 +159,7 @@ nsMimeBaseEmitter::CleanupHeaderArray(nsVoidArray *aArray)
   delete aArray;
 }
 
-static PRInt32 MapHeaderNameToID(const char *header)
+static int32_t MapHeaderNameToID(const char *header)
 {
   // emitter passes UPPERCASE for header names
   if (!strcmp(header, "DATE"))
@@ -225,7 +225,7 @@ nsMimeBaseEmitter::MimeGetStringByName(const char *aHeaderName)
                                                   getter_Copies(val));
 
     if (NS_FAILED(res))
-      return nsnull;
+      return nullptr;
 
     // Here we need to return a new copy of the string
     // This returns a UTF-8 string so the caller needs to perform a conversion
@@ -235,12 +235,12 @@ nsMimeBaseEmitter::MimeGetStringByName(const char *aHeaderName)
   }
   else
   {
-    return nsnull;
+    return nullptr;
   }
 }
 
 char *
-nsMimeBaseEmitter::MimeGetStringByID(PRInt32 aID)
+nsMimeBaseEmitter::MimeGetStringByID(int32_t aID)
 {
   nsresult res = NS_OK;
 
@@ -261,12 +261,12 @@ nsMimeBaseEmitter::MimeGetStringByID(PRInt32 aID)
     res = m_stringBundle->GetStringFromID(aID, getter_Copies(val));
 
     if (NS_FAILED(res))
-      return nsnull;
+      return nullptr;
 
     return ToNewUTF8String(val);
   }
   else
-    return nsnull;
+    return nullptr;
 }
 
 //
@@ -278,14 +278,14 @@ nsMimeBaseEmitter::MimeGetStringByID(PRInt32 aID)
 char *
 nsMimeBaseEmitter::LocalizeHeaderName(const char *aHeaderName, const char *aDefaultName)
 {
-  char *retVal = nsnull;
+  char *retVal = nullptr;
 
   // prefer to use translated strings if not for quoting
   if (mFormat != nsMimeOutput::nsMimeMessageQuoting &&
       mFormat != nsMimeOutput::nsMimeMessageBodyQuoting)
   {
     // map name to id and get the translated string
-    PRInt32 id = MapHeaderNameToID(aHeaderName);
+    int32_t id = MapHeaderNameToID(aHeaderName);
     if (id > 0)
       retVal = MimeGetStringByID(id);
   }
@@ -315,7 +315,7 @@ nsMimeBaseEmitter::SetPipe(nsIInputStream * aInputStream, nsIOutputStream *outSt
 // anything to the stream since these may be image data
 // output streams, etc...
 NS_IMETHODIMP
-nsMimeBaseEmitter::Initialize(nsIURI *url, nsIChannel * aChannel, PRInt32 aFormat)
+nsMimeBaseEmitter::Initialize(nsIURI *url, nsIChannel * aChannel, int32_t aFormat)
 {
   // set the url
   mURL = url;
@@ -379,7 +379,7 @@ nsMimeBaseEmitter::EndAttachment()
   if ( (mCurrentAttachment) && (mAttachArray) )
   {
     mAttachArray->AppendElement(mCurrentAttachment);
-    mCurrentAttachment = nsnull;
+    mCurrentAttachment = nullptr;
   }
 
   return NS_OK;
@@ -402,7 +402,7 @@ nsMimeBaseEmitter::UtilityWrite(const char *buf)
 {
   NS_ENSURE_ARG_POINTER(buf);
 
-  PRUint32    written;
+  uint32_t    written;
   Write(nsDependentCString(buf), &written);
   return NS_OK;
 }
@@ -410,7 +410,7 @@ nsMimeBaseEmitter::UtilityWrite(const char *buf)
 NS_IMETHODIMP
 nsMimeBaseEmitter::UtilityWrite(const nsACString &buf)
 {
-  PRUint32    written;
+  uint32_t    written;
   Write(buf, &written);
   return NS_OK;
 }
@@ -420,25 +420,25 @@ nsMimeBaseEmitter::UtilityWriteCRLF(const char *buf)
 {
   NS_ENSURE_ARG_POINTER(buf);
 
-  PRUint32    written;
+  uint32_t    written;
   Write(nsDependentCString(buf), &written);
   Write(NS_LITERAL_CSTRING(CRLF), &written);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsMimeBaseEmitter::Write(const nsACString &buf, PRUint32 *amountWritten)
+nsMimeBaseEmitter::Write(const nsACString &buf, uint32_t *amountWritten)
 {
   unsigned int        written = 0;
   nsresult rv = NS_OK;
-  PRUint32            needToWrite;
+  uint32_t            needToWrite;
 
 #ifdef DEBUG_BenB
   // If you want to see libmime output...
   printf("%s", buf);
 #endif
 
-  PR_LOG(gMimeEmitterLogModule, PR_LOG_ALWAYS, (PromiseFlatCString(buf).get()));
+  PR_LOG(gMimeEmitterLogModule, PR_LOG_ALWAYS, ("%s", PromiseFlatCString(buf).get()));
   //
   // Make sure that the buffer we are "pushing" into has enough room
   // for the write operation. If not, we have to buffer, return, and get
@@ -481,17 +481,18 @@ nsMimeBaseEmitter::Write(const nsACString &buf, PRUint32 *amountWritten)
 }
 
 nsresult
-nsMimeBaseEmitter::WriteHelper(const char *buf, PRUint32 count, PRUint32 *countWritten)
+nsMimeBaseEmitter::WriteHelper(const char *buf, uint32_t count, uint32_t *countWritten)
 {
   NS_ENSURE_TRUE(mOutStream, NS_ERROR_NOT_INITIALIZED);
 
   nsresult rv = mOutStream->Write(buf, count, countWritten);
   if (rv == NS_BASE_STREAM_WOULD_BLOCK) {
     // pipe is full, push contents of pipe to listener...
-    PRUint32 avail;
+    uint64_t avail;
     rv = mInputStream->Available(&avail);
     if (NS_SUCCEEDED(rv) && avail) {
-      mOutListener->OnDataAvailable(mChannel, mURL, mInputStream, 0, avail);
+      mOutListener->OnDataAvailable(mChannel, mURL, mInputStream, 0, 
+                                    NS_MIN(avail, PR_UINT32_MAX));
 
       // try writing again...
       rv = mOutStream->Write(buf, count, countWritten);
@@ -506,12 +507,12 @@ nsMimeBaseEmitter::WriteHelper(const char *buf, PRUint32 count, PRUint32 *countW
 const char *
 nsMimeBaseEmitter::GetHeaderValue(const char  *aHeaderName)
 {
-  PRInt32     i;
-  char        *retVal = nsnull;
+  int32_t     i;
+  char        *retVal = nullptr;
   nsVoidArray *array = mDocHeader? mHeaderArray : mEmbeddedHeaderArray;
 
   if (!array)
-    return nsnull;
+    return nullptr;
 
   for (i = 0; i < array->Count(); i++)
   {
@@ -698,7 +699,9 @@ nsMimeBaseEmitter::GenerateDateString(const char * dateString,
     dateFormatPrefs->SetBoolPref("date_senders_timezone", true);
 
   PRExplodedTime explodedMsgTime;
-  rv = PR_ParseTimeStringToExplodedTime(dateString, false, &explodedMsgTime);
+  // XXX Casting PRStatus to nsresult
+  rv = static_cast<nsresult>(
+    PR_ParseTimeStringToExplodedTime(dateString, false, &explodedMsgTime));
   /**
    * To determine the date format to use, comparison of current and message
    * time has to be made. If displaying in local time, both timestamps have
@@ -732,7 +735,7 @@ nsMimeBaseEmitter::GenerateDateString(const char * dateString,
   nsAutoString formattedDateString;
   if (NS_SUCCEEDED(rv))
   {
-    rv = mDateFormatter->FormatPRExplodedTime(nsnull /* nsILocale* locale */,
+    rv = mDateFormatter->FormatPRExplodedTime(nullptr /* nsILocale* locale */,
                                               dateFormat,
                                               kTimeFormatNoSeconds,
                                               &explodedCompTime,
@@ -743,7 +746,7 @@ nsMimeBaseEmitter::GenerateDateString(const char * dateString,
       if (displaySenderTimezone)
       {
         // offset of local time from UTC in minutes
-        PRInt32 senderoffset = (explodedMsgTime.tm_params.tp_gmt_offset +
+        int32_t senderoffset = (explodedMsgTime.tm_params.tp_gmt_offset +
                                 explodedMsgTime.tm_params.tp_dst_offset) / 60;
         // append offset to date string
         PRUnichar *tzstring =
@@ -764,7 +767,7 @@ nsMimeBaseEmitter::GenerateDateString(const char * dateString,
 char*
 nsMimeBaseEmitter::GetLocalizedDateString(const char * dateString)
 {
-  char *i18nValue = nsnull;
+  char *i18nValue = nullptr;
 
   bool displayOriginalDate = false;
   nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID));
@@ -791,8 +794,8 @@ nsMimeBaseEmitter::GetLocalizedDateString(const char * dateString)
 nsresult
 nsMimeBaseEmitter::WriteHeaderFieldHTML(const char *field, const char *value)
 {
-  char *newValue = nsnull;
-  char *i18nValue = nsnull;
+  char *newValue = nullptr;
+  char *i18nValue = nullptr;
 
   if ( (!field) || (!value) )
     return NS_OK;
@@ -819,7 +822,7 @@ nsMimeBaseEmitter::WriteHeaderFieldHTML(const char *field, const char *value)
 
     // we're going to need a converter to convert
     nsresult rv = mUnicodeConverter->DecodeMimeHeaderToCharPtr(
-      i18nValue, nsnull, false, true, getter_Copies(tValue));
+      i18nValue, nullptr, false, true, getter_Copies(tValue));
     if (NS_SUCCEEDED(rv) && !tValue.IsEmpty())
       newValue = MsgEscapeHTML(tValue.get());
     else
@@ -987,7 +990,7 @@ nsMimeBaseEmitter::DumpToCC()
 nsresult
 nsMimeBaseEmitter::DumpRestOfHeaders()
 {
-  PRInt32     i;
+  int32_t     i;
   nsVoidArray *array = mDocHeader? mHeaderArray : mEmbeddedHeaderArray;
 
   mHTMLHeaders.Append("<table border=0 cellspacing=0 cellpadding=0 width=\"100%\" class=\"header-part3\">");
@@ -1042,7 +1045,7 @@ nsMimeBaseEmitter::Complete()
   // If we are here and still have data to write, we should try
   // to flush it...if we try and fail, we should probably return
   // an error!
-  PRUint32      written;
+  uint32_t      written;
 
   nsresult rv = NS_OK;
   while ( NS_SUCCEEDED(rv) && (mBufferMgr) && (mBufferMgr->GetSize() > 0))
@@ -1050,13 +1053,13 @@ nsMimeBaseEmitter::Complete()
 
   if (mOutListener)
   {
-    PRUint32 bytesInStream = 0;
+    uint64_t bytesInStream = 0;
     nsresult rv2 = mInputStream->Available(&bytesInStream);
     NS_ASSERTION(NS_SUCCEEDED(rv2), "Available failed");
     if (bytesInStream)
     {
       nsCOMPtr<nsIRequest> request = do_QueryInterface(mChannel);
-      rv2 = mOutListener->OnDataAvailable(request, mURL, mInputStream, 0, bytesInStream);
+      mOutListener->OnDataAvailable(request, mURL, mInputStream, 0, NS_MIN(bytesInStream, PR_UINT32_MAX));
     }
   }
 
@@ -1082,7 +1085,7 @@ nsMimeBaseEmitter::StartBody(bool bodyOnly, const char *msgID, const char *outCh
 }
 
 NS_IMETHODIMP
-nsMimeBaseEmitter::WriteBody(const nsACString &buf, PRUint32 *amountWritten)
+nsMimeBaseEmitter::WriteBody(const nsACString &buf, uint32_t *amountWritten)
 {
   return NS_OK;
 }
