@@ -4,12 +4,11 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 349482;
-var summary = 'Decompiling try/catch in with() should not crash';
+var BUGNUMBER = 452573;
+var summary = 'Do not assert with JIT: JSVAL_IS_VOID(boxed) || JSVAL_IS_BOOLEAN(boxed)';
 var actual = 'No Crash';
 var expect = 'No Crash';
 
-
 //-----------------------------------------------------------------------------
 test();
 //-----------------------------------------------------------------------------
@@ -19,9 +18,12 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
-  var f = function() { with({})     { try{}catch(e){} } }
-  print(f.toString());
+
+  jit(true);
+
+  for(var j=0;j<5;++j) typeof void /x/;
+
+  jit(false);
 
   reportCompare(expect, actual, summary);
 
