@@ -5,54 +5,44 @@
 
 package org.mozilla.gecko;
 
+import org.mozilla.gecko.AwesomeBar.ContextMenuSubject;
+import org.mozilla.gecko.db.BrowserContract.Combined;
+import org.mozilla.gecko.db.BrowserDB;
+import org.mozilla.gecko.db.BrowserDB.URLColumns;
+import org.mozilla.gecko.util.GeckoEventListener;
+
+import org.json.JSONArray;
+import org.json.JSONException;
+import org.json.JSONObject;
+
 import android.app.Activity;
 import android.content.Context;
-import android.widget.ListView;
-import android.widget.ImageView;
-import android.widget.TextView;
-import android.view.View;
-import android.view.ContextMenu;
-import android.view.ContextMenu.ContextMenuInfo;
-import android.view.MenuItem;
-import android.view.View.OnClickListener;
-import android.view.View.OnLongClickListener;
-import android.app.Activity;
-import android.widget.AdapterView;
 import android.database.Cursor;
-import android.widget.AdapterView;
-import android.util.Log;
-import android.text.TextUtils;
-import android.widget.Toast;
-import android.widget.SimpleCursorAdapter;
-import android.widget.LinearLayout;
-import android.widget.TabHost.TabContentFactory;
-import android.view.LayoutInflater;
-import android.view.ViewGroup;
 import android.graphics.drawable.Drawable;
-import android.graphics.Bitmap;
-import android.graphics.BitmapFactory;
-import android.content.Intent;
-import android.widget.FilterQueryProvider;
 import android.os.AsyncTask;
 import android.os.SystemClock;
-import android.view.MenuInflater;
-import android.widget.TabHost;
+import android.util.Log;
 import android.view.ContextMenu;
 import android.view.ContextMenu.ContextMenuInfo;
+import android.view.LayoutInflater;
+import android.view.MenuInflater;
+import android.view.View;
+import android.view.View.OnClickListener;
+import android.view.View.OnLongClickListener;
+import android.view.ViewGroup;
+import android.widget.AdapterView;
+import android.widget.FilterQueryProvider;
+import android.widget.ImageView;
+import android.widget.LinearLayout;
+import android.widget.ListView;
+import android.widget.SimpleCursorAdapter;
+import android.widget.TabHost.TabContentFactory;
+import android.widget.TextView;
 
 import java.io.ByteArrayInputStream;
 import java.io.IOException;
 import java.util.ArrayList;
 
-import org.json.JSONArray;
-import org.json.JSONObject;
-import org.json.JSONException;
-
-import org.mozilla.gecko.AwesomeBar.ContextMenuSubject;
-import org.mozilla.gecko.db.BrowserDB;
-import org.mozilla.gecko.db.BrowserDB.URLColumns;
-import org.mozilla.gecko.db.BrowserContract.Combined;
-
 public class AllPagesTab extends AwesomeBarTab implements GeckoEventListener {
     public static final String LOGTAG = "ALL_PAGES";
     private static final String TAG = "allPages";
@@ -77,7 +67,7 @@ public class AllPagesTab extends AwesomeBarTab implements GeckoEventListener {
         super(context);
         mSearchEngines = new ArrayList<SearchEngine>();
 
-        GeckoAppShell.registerGeckoEventListener("SearchEngines:Data", this);
+        registerEventListener("SearchEngines:Data");
         GeckoAppShell.sendEventToGecko(GeckoEvent.createBroadcastEvent("SearchEngines:Get", null));
     }
 
@@ -121,7 +111,7 @@ public class AllPagesTab extends AwesomeBarTab implements GeckoEventListener {
 
     public void destroy() {
         AwesomeBarCursorAdapter adapter = getCursorAdapter();
-        GeckoAppShell.unregisterGeckoEventListener("SearchEngines:Data", this);
+        unregisterEventListener("SearchEngines:Data");
         if (adapter == null) {
             return;
         }
@@ -135,6 +125,10 @@ public class AllPagesTab extends AwesomeBarTab implements GeckoEventListener {
         AwesomeBarCursorAdapter adapter = getCursorAdapter();
         adapter.filter(searchTerm);
 
+        filterSuggestions(searchTerm);
+    }
+
+    private void filterSuggestions(String searchTerm) {
         // cancel previous query
         if (mSuggestTask != null) {
             mSuggestTask.cancel(true);
@@ -178,6 +172,7 @@ public class AllPagesTab extends AwesomeBarTab implements GeckoEventListener {
 
     private interface AwesomeBarItem {
         public void onClick();
+        public ContextMenuSubject getSubject();
     }
 
     private class AwesomeBarCursorItem implements AwesomeBarItem {
@@ -187,10 +182,6 @@ public class AllPagesTab extends AwesomeBarTab implements GeckoEventListener {
             mCursor = cursor;
         }
 
-        public Cursor getCursor() {
-            return mCursor;
-        }
-
         public void onClick() {
             AwesomeBarTabs.OnUrlOpenListener listener = getUrlListener();
             if (listener == null)
@@ -199,6 +190,22 @@ public class AllPagesTab extends AwesomeBarTab implements GeckoEventListener {
             String url = mCursor.getString(mCursor.getColumnIndexOrThrow(URLColumns.URL));
             listener.onUrlOpen(url);
         }
+
+        public ContextMenuSubject getSubject() {
+            // Use the history id in order to allow removing history entries
+            int id = mCursor.getInt(mCursor.getColumnIndexOrThrow(Combined.HISTORY_ID));
+
+            String keyword = null;
+            int keywordCol = mCursor.getColumnIndex(URLColumns.KEYWORD);
+            if (keywordCol != -1)
+                keyword = mCursor.getString(keywordCol);
+
+            return new ContextMenuSubject(id,
+                                          mCursor.getString(mCursor.getColumnIndexOrThrow(URLColumns.URL)),
+                                          mCursor.getBlob(mCursor.getColumnIndexOrThrow(URLColumns.FAVICON)),
+                                          mCursor.getString(mCursor.getColumnIndexOrThrow(URLColumns.TITLE)),
+                                          keyword);
+        }
     }
 
     private class AwesomeBarSearchEngineItem implements AwesomeBarItem {
@@ -213,6 +220,11 @@ public class AllPagesTab extends AwesomeBarTab implements GeckoEventListener {
             if (listener != null)
                 listener.onSearch(mSearchEngine, mSearchTerm);
         }
+
+        public ContextMenuSubject getSubject() {
+            // Do not show context menu for search engine items
+            return null;
+        }
     }
 
     private class AwesomeBarCursorAdapter extends SimpleCursorAdapter {
@@ -468,7 +480,8 @@ public class AllPagesTab extends AwesomeBarTab implements GeckoEventListener {
             Log.e(LOGTAG, "Error getting search engine JSON", e);
         }
 
-        filter(mSearchTerm);
+        mCursorAdapter.notifyDataSetChanged();
+        filterSuggestions(mSearchTerm);
     }
 
     private Drawable getDrawableFromDataURI(String dataURI) {
@@ -533,29 +546,7 @@ public class AllPagesTab extends AwesomeBarTab implements GeckoEventListener {
 
         ListView list = (ListView)view;
         AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo) menuInfo;
-        Object selectedItem = list.getItemAtPosition(info.position);
-
-        if (!(selectedItem instanceof AwesomeBarCursorItem)) {
-            Log.e(LOGTAG, "item at " + info.position + " is a search item");
-            return subject;
-        }
-
-        Cursor cursor = ((AwesomeBarCursorItem) selectedItem).getCursor();
-
-        // Don't show the context menu for folders
-        String keyword = null;
-        int keywordCol = cursor.getColumnIndex(URLColumns.KEYWORD);
-        if (keywordCol != -1)
-            keyword = cursor.getString(keywordCol);
-
-        // Use the history id in order to allow removing history entries
-        int id = cursor.getInt(cursor.getColumnIndexOrThrow(Combined.HISTORY_ID));
-
-        subject = new ContextMenuSubject(id,
-                                        cursor.getString(cursor.getColumnIndexOrThrow(URLColumns.URL)),
-                                        cursor.getBlob(cursor.getColumnIndexOrThrow(URLColumns.FAVICON)),
-                                        cursor.getString(cursor.getColumnIndexOrThrow(URLColumns.TITLE)),
-                                        keyword);
+        subject = ((AwesomeBarItem) list.getItemAtPosition(info.position)).getSubject();
 
         if (subject == null)
             return subject;
@@ -572,4 +563,12 @@ public class AllPagesTab extends AwesomeBarTab implements GeckoEventListener {
         menu.setHeaderTitle(subject.title);
         return subject;
     }
+
+    private void registerEventListener(String event) {
+        GeckoAppShell.getEventDispatcher().registerEventListener(event, this);
+    }
+
+    private void unregisterEventListener(String event) {
+        GeckoAppShell.getEventDispatcher().unregisterEventListener(event, this);
+    }
 }
