@@ -15,7 +15,7 @@ class nsIdleServiceAndroid : public nsIdleService
 public:
     NS_DECL_ISUPPORTS_INHERITED
 
-    bool PollIdleTime(PRUint32* aIdleTime);
+    bool PollIdleTime(uint32_t* aIdleTime);
 
     static already_AddRefed<nsIdleServiceAndroid> GetInstance() 
     {
