@@ -24,7 +24,7 @@ class nsIdleServiceWin : public nsIdleService
 public:
     NS_DECL_ISUPPORTS_INHERITED
 
-    bool PollIdleTime(PRUint32* aIdleTime);
+    bool PollIdleTime(uint32_t* aIdleTime);
 
     static already_AddRefed<nsIdleServiceWin> GetInstance()
     {
